=== added directory 'plugin/zeromq'
=== added file 'plugin/zeromq/plugin.ac'
--- plugin/zeromq/plugin.ac	1970-01-01 00:00:00 +0000
+++ plugin/zeromq/plugin.ac	2011-08-08 11:01:35 +0000
@@ -0,0 +1,8 @@
+AC_LIB_HAVE_LINKFLAGS(zmq,,[
+    #include <zmq.h>
+  ],[
+	 void *context = zmq_init (1);
+  ])
+AS_IF([test "x$ac_cv_libzmq" = "xno"],[
+    AC_MSG_WARN([zeromq not found: not building zmq plugin.])
+])

=== added file 'plugin/zeromq/plugin.cnf'
--- plugin/zeromq/plugin.cnf	1970-01-01 00:00:00 +0000
+++ plugin/zeromq/plugin.cnf	2011-08-08 11:01:35 +0000
@@ -0,0 +1,2 @@
+plugin-add=zeromq
+

=== added file 'plugin/zeromq/plugin.ini'
--- plugin/zeromq/plugin.ini	1970-01-01 00:00:00 +0000
+++ plugin/zeromq/plugin.ini	2011-08-08 11:01:35 +0000
@@ -0,0 +1,14 @@
+[plugin]
+name=zeromq
+version=0.1
+author=Marcus Eriksson
+license=PLUGIN_LICENSE_GPL
+title=ZeroMQ Replicator
+description=Replicator plugin that publishes transactions to ZeroMQ
+load_by_default=no
+sources=zeromq_log.cc
+headers=zeromq_log.h
+cxxflags=${PROTOSKIP_WARNINGS}
+build_conditional="$ac_cv_libzmq" = "yes"
+ldflags=${LTLIBZMQ}
+

=== added file 'plugin/zeromq/zeromq_log.cc'
--- plugin/zeromq/zeromq_log.cc	1970-01-01 00:00:00 +0000
+++ plugin/zeromq/zeromq_log.cc	2011-08-08 11:01:35 +0000
@@ -0,0 +1,119 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 Marcus Eriksson
+ *
+ *  Authors:
+ *
+ *  Marcus Eriksson <krummas@gmail.com>
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <config.h>
+#include "zeromq_log.h"
+#include <drizzled/message/transaction.pb.h>
+#include <google/protobuf/io/coded_stream.h>
+#include <stdio.h>
+#include <drizzled/module/registry.h>
+#include <drizzled/plugin.h>
+#include <stdint.h>
+#include <boost/program_options.hpp>
+#include <drizzled/module/option_map.h>
+#include <zmq.h>
+
+namespace po= boost::program_options;
+
+using namespace std;
+using namespace drizzled;
+using namespace google;
+
+namespace drizzle_plugin
+{
+
+ZeroMQLog::ZeroMQLog(const string &name, const string &endpoint) :
+  plugin::TransactionApplier(name)
+{
+  void *context= zmq_init(1);
+  _socket= zmq_socket (context, ZMQ_PUB);
+  assert (_socket);
+  int rc= zmq_bind (_socket, endpoint.c_str());
+  assert (rc == 0);
+  pthread_mutex_init(&publishLock, NULL);
+}
+
+ZeroMQLog::~ZeroMQLog()
+{
+  zmq_close(_socket);
+  pthread_mutex_destroy(&publishLock);
+}
+
+plugin::ReplicationReturnCode
+ZeroMQLog::apply(Session &, const message::Transaction &to_apply)
+{
+  size_t message_byte_length= to_apply.ByteSize();
+  uint8_t* buffer= new uint8_t[message_byte_length];
+  if(buffer == NULL)
+  {
+    errmsg_printf(error::ERROR, _("Failed to allocate enough memory to transaction message\n"));
+    deactivate();
+    return plugin::UNKNOWN_ERROR;
+  }
+
+  to_apply.SerializeWithCachedSizesToArray(buffer);
+  zmq_msg_t msg;
+  int rc= zmq_msg_init_size (&msg, message_byte_length);
+  assert (rc == 0);
+  memcpy(zmq_msg_data(&msg), buffer, message_byte_length);
+
+  pthread_mutex_lock(&publishLock);
+  rc= zmq_send (_socket, &msg, 0);
+  pthread_mutex_unlock(&publishLock);
+
+  zmq_msg_close (&msg);
+  delete[] buffer;
+  return plugin::SUCCESS;
+}
+
+static ZeroMQLog *zeromqLogger; ///< the actual plugin
+
+/**
+ * Initialize the zeromq logger
+ */
+static int init(drizzled::module::Context &context)
+{
+  const module::option_map &vm= context.getOptions();
+  zeromqLogger= new ZeroMQLog("zeromq_log_applier", vm["endpoint"].as<string>());
+  context.add(zeromqLogger);
+  ReplicationServices::attachApplier(zeromqLogger, vm["use-replicator"].as<string>());
+  context.registerVariable(new sys_var_const_string_val("endpoint", vm["endpoint"].as<string>()));
+  return 0;
+}
+
+
+static void init_options(drizzled::module::option_context &context)
+{
+  context("endpoint", 
+          po::value<string>()->default_value("tcp://*:9999"),
+          _("End point to bind to"));
+  context("use-replicator",
+          po::value<string>()->default_value("default_replicator"),
+          _("Name of the replicator plugin to use (default='default_replicator')"));
+
+}
+
+} /* namespace drizzle_plugin */
+
+DRIZZLE_PLUGIN(drizzle_plugin::init, NULL, drizzle_plugin::init_options);

=== added file 'plugin/zeromq/zeromq_log.h'
--- plugin/zeromq/zeromq_log.h	1970-01-01 00:00:00 +0000
+++ plugin/zeromq/zeromq_log.h	2011-08-08 11:01:35 +0000
@@ -0,0 +1,78 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 Marcus Eriksson
+ *
+ *  Authors:
+ *
+ *  Marcus Eriksson <krummas@gmail.com>
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+
+#pragma once
+
+#include <drizzled/replication_services.h>
+#include <drizzled/plugin/transaction_applier.h>
+#include <string>
+#include <zmq.h>
+
+namespace drizzle_plugin
+{
+
+/**
+ * @brief
+ *   A TransactionApplier that publishes transaction on a zeromq pub socket
+ *
+ * @details
+ * sends messages to a pub socket
+ */
+class ZeroMQLog :
+  public drizzled::plugin::TransactionApplier 
+{
+private:
+  void *_socket;
+  pthread_mutex_t publishLock;
+public:
+
+  /**
+   * @brief
+   *   Constructs a new ZeroMQLog.
+   *
+   * @details
+   *
+   * @param[in] name of plugin
+   * @param[in] the endpoint string (typically tcp:// *:9999) (no space actually, compiler whines about comment within comment..)
+  */
+  ZeroMQLog(const std::string &name, const std::string &endpoint);
+  ~ZeroMQLog();
+
+  /**
+   * @brief
+   *   Serializes the transaction and publishes the message to zmq
+   *
+   * @details
+   *   Serializes the protobuf transaction and drops it on zmq
+   *
+   * @param[in] to_apply the transaction to send
+   */
+  drizzled::plugin::ReplicationReturnCode
+  apply(drizzled::Session &session, const drizzled::message::Transaction &to_apply);
+
+};
+
+} /* namespace drizzle_plugin */
+

